@@ -56,9 +56,7 @@ class ComponentState(dict):
 
     def __setitem__(self, key, value):
         if not key in self.keys():
-            raise JembeError(
-                "Can't add new param '{}' to component state {}".format(key, self)
-            )
+            raise JembeError(f"Can't add new param '{key}' to component state {self}")
         return super().__setitem__(key, value)
 
     def __delitem__(self, key):
@@ -133,7 +131,7 @@ class ComponentReference:
         ):  # a/b ../b
             if name_split[0] == "":
                 name_split = name_split[1:]
-                name_split[0] = "/{}".format(name_split[0])
+                name_split[0] = f"/{name_split[0]}"
             for pname in name_split[:-1]:
                 if cr is None:
                     cr = cls(
@@ -188,7 +186,8 @@ class ComponentReference:
         self.action = ComponentConfig.DEFAULT_DISPLAY_ACTION
         self.action_args: Tuple[Any, ...] = ()
         self.action_kwargs: dict = {}
-        self.kwargs = {k: v for k, v in kwargs.items() if not k.startswith("_")}
+        self.state_kwargs = {k: v for k, v in kwargs.items() if not k.startswith("_")}
+        self.kwargs = {k: v for k, v in kwargs.items()}
         self.merge_existing_params = merge_existing_params
 
         self._component_initialise_done = False
@@ -211,6 +210,7 @@ class ComponentReference:
 
         self.root_renderer = self
         self.active_renderer = self
+        self.parent_reference = self
         self._aditional_components: List["Component"] = []
         self.base_jrl = "$jmb"
 
@@ -222,6 +222,7 @@ class ComponentReference:
         )
         cr.root_renderer = self.root_renderer
         cr.root_renderer.active_renderer = cr
+        cr.parent_reference = self
         cr._aditional_components = self._aditional_components.copy()
         if (
             self.component_instance is not None
@@ -234,9 +235,7 @@ class ComponentReference:
     def component(self, jmb_exec_name: str, **kwargs) -> "jembe.ComponentReference":
         if self.action != ComponentConfig.DEFAULT_DISPLAY_ACTION:
             raise JembeError(
-                "<{}> Cant reference child component when action '{}' is called".format(
-                    str(self), self.action
-                )
+                f"<{str(self)}> Cant reference child component when action '{self.action}' is called"
             )
         return self._component(True, jmb_exec_name, **kwargs)
 
@@ -278,9 +277,7 @@ class ComponentReference:
             ci = self.component_instance
             if ci is None:
                 current_app.logger.warning(
-                    "Component '{}' can't acuire referenced component '{}' to check if action '{}' is accessible".format(
-                        self.root_renderer.caller_exec_name, self.exec_name, self.action
-                    )
+                    f"Component '{self.root_renderer.caller_exec_name}' can't acuire referenced component '{self.exec_name}' to check if action '{self.action}' is accessible"
                 )
                 return True
 
@@ -320,11 +317,15 @@ class ComponentReference:
             "component{reset}('{name}'{kwargs}{key})".format(
                 reset="_reset" if not self.merge_existing_params else "",
                 name=self.name,
-                key=",key='{}'".format(self._key) if self._key else "",
+                key=f",key='{self.key}'" if self._key else "",
                 kwargs=",{}".format(
-                    re.sub('(?<!\\\\)"', "'", dumps(self.kwargs, separators=(",", ":")))
+                    re.sub(
+                        '(?<!\\\\)"',
+                        "'",
+                        dumps(self.state_kwargs, separators=(",", ":")),
+                    )
                 )
-                if self.kwargs
+                if self.state_kwargs
                 else "",
             )
             if self.name != "."
@@ -355,7 +356,7 @@ class ComponentReference:
             if self.base_jrl.endswith(".display()")
             else self.base_jrl
         )
-        return Markup("{}.{}".format(base_jrl, jrl))
+        return Markup(f"{base_jrl}.{jrl}")
 
     @cached_property
     def exec_name(self) -> str:
@@ -383,20 +384,17 @@ class ComponentReference:
     def processor(self) -> "jembe.Processor":
         return get_processor()
 
-    def __call__(self) -> str:
-        """
-        Do render
-
-        Component initialisation must be executed in place in order to know
-        will it raise exception (like NotFound, Forbidden, Unauthorized etc.)
-        so that we can decide how to render template
+    def execute(self):
+        """Calls all necessary init, display and call components to execute this component reference
+        inside current jembe processor.
         """
         if self.exec_name in self.processor.components_marked_for_removal:
             raise JembeError(
-                "Cant display component '{}' marked for removal in parent component template!".format(
-                    self.exec_name
-                )
+                f"Cant display component '{self.exec_name}' marked for removal in parent component template!"
             )
+        if self.parent_reference != self:
+            self.parent_reference.execute()
+
         self.processor.add_command(
             InitialiseCommand(self.exec_name, self.kwargs, self.merge_existing_params),
             end=True,
@@ -425,9 +423,17 @@ class ComponentReference:
                 ),
                 end=True,
             )
-        return Markup(
-            '<template jmb-placeholder="{}"></template>'.format(self.exec_name)
-        )
+
+    def __call__(self) -> str:
+        """
+        Do render
+
+        Component initialisation must be executed in place in order to know
+        will it raise exception (like NotFound, Forbidden, Unauthorized etc.)
+        so that we can decide how to render template
+        """
+        self.execute()
+        return Markup(f'<template jmb-placeholder="{self.exec_name}"></template>')
 
     def key(self, key: str) -> "jembe.ComponentReference":
         self._key = key
@@ -470,9 +476,7 @@ def componentInitDecorator(init_method):
             for name, value in params_to_inject.items():
                 if current_app.debug and name in kwargs:
                     current_app.logger.warning(
-                        "Injecting already set param {} in component {}".format(
-                            name, self._config.full_name
-                        )
+                        f"Injecting already set param {name} in component {self._config.full_name}"
                     )
                 kwargs[name] = value
 
@@ -490,9 +494,7 @@ def componentInitDecorator(init_method):
             init_method(self, *args, **kwargs)
         except Exception as e:
             current_app.logger.warning(
-                "{}: {};  args={}; kwargs={};".format(
-                    self.__class__.__name__, e, args, kwargs
-                )
+                f"{self.__class__.__name__}: {e};  args={args}; kwargs={kwargs};"
             )
             raise e
 
@@ -540,6 +542,115 @@ class ComponentMeta(ABCMeta):
 
 
 class Component(metaclass=ComponentMeta):
+    """Component is the primary building block for creating Web Applications using Jembe Framework.
+
+    Component is responsible:
+        - for displaying part of the web page, and
+        - handling all user interaction with that part of the page.
+
+    Component is made from two parts:
+        - Python class that holds and executes application logic, and
+        - associated Jinja template that renderes, component in browser;
+
+
+    Component can:
+        - be comined with other components in nested hirearchy to form web application;
+        - execute "actions" when end user interacts with component in browser;
+        - dispatch events to other components in application;
+        - listen for events dispatched to them;
+
+    .. note::
+
+        Component behavior can be configured with Component.Config class that is initialised only once when the application starts.
+
+    Component has **State variables**. State variables defines component state. Two instance of same component are
+    equal if they are having equal state variables.
+
+
+    - State variable are passed to component on its initialisation;
+    - Same component with same values of state variables are considered equal (thay should display exacly same HTML in browser)
+    - When value of any state variable is changed component will redisplay itself;
+    - State variables are defined as arguments to __init__ method.
+    - State varibales must have type annotation, and type annotation cannot be in double quotes.
+    - State variables are directly accessible to JavaScript in browser;
+    - State variables can be changed by JavaScript in browser cosing component to redisplay itself.
+
+    .. code-block:: python
+
+        @jmb.page('hello')
+        class HelloWorld(Component):
+            def __init__(self, name:str = "World"):
+                ''' name is state variable '''
+                pass
+
+    .. code-block:: html
+
+        <html><body>
+            <h1>Hello {{name}}</h1>
+            <input type="text" value="{{name}}" jmb-on:keydown.debounce="name = $self.value">
+
+            <script src="{{ url_for('jembe.static', filename='js/jembe.js') }}"></script>
+        </body></html>
+
+    .. figure:: /img/hello_world.gif
+        :alt: Hello World
+
+    **Component __init__ arguments**
+
+    Component init argumets are used to define:
+
+    - **state variables** and,
+    - performance variables.
+
+    **State variables** are __init__ arguments that does not start with '_' (underscore).
+    State variables defines the state of the component. Two instance of the same component in the same state are
+    considered equal and thay should behaive in the exacly same way.
+
+    **Performance variables** are __init__ arguments whose name starts with underscore '_'.
+    Performance variables are used to avoid unnecesary calculations or call to database, when parent component
+    has already calculated or aquiried value needed by component.
+
+    State variables without default values will become **URL arguments**. Value of Url arguments
+    must be provided as part of URL (for example: https://your_site/project/project/1).
+    State variables without default values, can be one of following types: int, str, URI.
+
+    .. code-block:: python
+
+        @jmb.page('demo')
+        class CDemo(Component):
+            def __init__(self, id: int, find:Optional[str] = None, _record:Optional[dict] = None):
+                '''
+                    - 'id' is State variable of type int.
+                    - 'id' is also URL variable because it does not have default value.
+                      When browser access:
+
+                      -  https://your_site/demo/1 id will have value of 1
+                      -  https://your_site/demo/2 id will have value of 2, and so on
+
+                    - 'find' is state variable of type optional str with default value of None.
+                      This variable can't be set via url like 'id'.
+                    - '_record' is performance variable it is not state variable.
+                '''
+                if record['id'] = id:
+                    # if _record is valid use it
+                    self._record = record
+
+            @property
+            def record(self):
+                try:
+                    return self._record
+                except:
+                    self._record = db.session.query(Record).get(self.state.id)
+                    return self._record
+
+
+    Args:
+        metaclass (_type_, optional): _description_. Defaults to ComponentMeta.
+
+    Raises:
+        JembeError: When something goes wrong with component initialisation
+    """
+
     @classmethod
     def _jembe_init_(
         cls,
@@ -548,7 +659,7 @@ class Component(metaclass=ComponentMeta):
         _jembe_injected_params_names: List[str],
         _jembe_merged_existing_params: bool,
         _jembe_existing_component_state: Optional["ComponentState"],
-        **init_params
+        **init_params,
     ):
         """
         Instance creation by explicitly calling __new__ and __init__
@@ -585,6 +696,19 @@ class Component(metaclass=ComponentMeta):
     _jembe_disabled_actions: List[str]
 
     class Config(ComponentConfig):
+        """Compononent config defines behavior of all instances of component, with what is
+        known at build time, like: url_path, subcomponents, name etc.
+
+        Component can access his config instance using ``self._config``.
+
+        Args:
+            template (Optional[Union[str, Iterable[str]]], optional): Path to default template for displaying component. If not provided template name will be same as ``self.full_name`` with '.html' extension added. Defaults to None.
+            components (Optional[Dict[str, jembe.ComponentRef]], optional): Subcomponent definitions. Defaults to None.
+            inject_into_components (Optional[ Callable[[jembe.Component, jembe.ComponentConfig], dict] ], optional): Callable to inject __init__ paramas into subcomponents. Defaults to None.
+            changes_url (bool, optional): Does this componet changes location URL when displayed on page. Defaults to True.
+            url_query_params (Optional[Dict[str, str]], optional): Mapping from GET Query params to state variables allowing state variables to be set with GET Query params (?var1=value&var2=value).dict(<name of get queryparam> = <name of state variable>) . Defaults to None.
+        """
+
         pass
 
     def __init__(self):
@@ -599,6 +723,9 @@ class Component(metaclass=ComponentMeta):
         """
         Normalised __init__ without params definition.
 
+        init is called after __init__ so it can access all state variables
+        regardles if __init__ is overwriten.
+
         Executed after __init__, usefull for adding logic that should be
         executed on init but without need to rewrite/list init params.
         """
@@ -608,6 +735,10 @@ class Component(metaclass=ComponentMeta):
 
     @property
     def previous_state(self) -> Optional["ComponentState"]:
+        """Returns previous state of the component when component state is changed in browser using javascript.
+
+        Accesible only in ``__init__`` and ``init`` methods.
+        """
         if not self._jembe_component_initialising:
             raise ComponentPreviousStateUnavaiableError(
                 "Component previous state is only avaiable during "
@@ -617,16 +748,39 @@ class Component(metaclass=ComponentMeta):
 
     @property
     def key(self) -> str:
+        """Returns component key.
+
+        ``key`` can be set by parent component, allowing us to have multiple
+        instance of same component in different state. Parent component
+        must make sure that there is not two component with same key.
+
+        Usefull when displaying list of records with cards.
+
+        If we try to display multiple components in diferent state without setting key,
+        only last component will be displayed overwriting all previous ones.
+
+        Returns:
+            str: component key
+        """
         return self.__key
 
     @key.setter
     def key(self, key: str):
         self.__key = key
+        # self.__dict__.pop("key", None) # delete cached property
         # TODO update __exec_name
         # self.__update_exec_name()
 
     @property
     def exec_name(self) -> str:
+        """Uniquly identifies instance of the Component in application.
+
+        It is created from component ``full_name`` adding all ``keys`` where necessary
+        (/component1/component2.key2/component3/component4.key4).
+
+        Returns:
+            str: Component Execution Name
+        """
         return self.__exec_name
 
     @exec_name.setter
@@ -634,7 +788,7 @@ class Component(metaclass=ComponentMeta):
         # verify exec_name with _config.full_name
         if exec_name_to_full_name(exec_name) != self._config.full_name:
             raise JembeError(
-                "Invalid exec_name {} for {}".format(exec_name, self._config.full_name)
+                f"Invalid exec_name {exec_name} for {self._config.full_name}"
             )
 
         self.__exec_name = exec_name
@@ -657,9 +811,7 @@ class Component(metaclass=ComponentMeta):
         self.__has_action_or_listener_executed = True
 
     def set_parent_exec_name(self, parent_exec_name: str):
-        self.__exec_name = "{}/{}.{}".format(
-            parent_exec_name, self._config.name, self.key
-        )
+        self.__exec_name = f"{parent_exec_name}/{self._config.name}.{self.key}"
         # TODO verify parent_exec_name with _config.full_name
         # TODO set __exec_name
 
@@ -668,26 +820,24 @@ class Component(metaclass=ComponentMeta):
         cls, name: str, key: str = "", parent_exec_name: str = ""
     ) -> str:
         """Build component exec name"""
-        local_exec_name = name if not key else "{}.{}".format(name, key)
+        local_exec_name = name if not key else f"{name}.{key}"
         return "/".join((parent_exec_name, local_exec_name))
 
     @property
     def url(self) -> str:
-        """
-        Returns url of this component build using url_path of parent
-        components and url_path of this component
+        """Returns url of this component.
+
+        Url is build using url_path of parent components and url_path of this component
         """
         url = self._config.build_url(self.exec_name, self)
         url_get_params = []
         for url_param_name, state_param_name in self._config.url_query_params.items():
             if self.state.get(state_param_name, None) is not None:
                 url_get_params.append(
-                    "{}={}".format(
-                        url_param_name, quote_plus(str(self.state[state_param_name]))
-                    )
+                    f"{url_param_name}={quote_plus(str(self.state[state_param_name]))}"
                 )
         if url_get_params:
-            url = "{}?{}".format(url, "&".join(url_get_params))
+            url = f"{url}?{'&'.join(url_get_params)}"
         return url
 
     @classmethod
@@ -710,20 +860,13 @@ class Component(metaclass=ComponentMeta):
             except Exception as e:
                 if current_app.debug or current_app.testing:
                     raise JembeError(
-                        "State param {} of {}.{} with hint {} is not supported for json dump/load "
-                        "nor custom encoding/decoding logic is defined in dump_init_param/load_init_param.\n ({})\n\n "
-                        "Please double check that you are using actuall class in annotation and not string with class name.".format(
-                            name,
-                            cls.__module__,
-                            cls.__name__,
-                            cls._jembe_init_signature.parameters[name],
-                            e,
-                        )
+                        f"State param {name} of {cls.__module__}.{cls.__name__} with hint {cls._jembe_init_signature.parameters[name]} is not supported for json dump/load "
+                        f"nor custom encoding/decoding logic is defined in dump_init_param/load_init_param.\n ({e})\n\n "
                     )
         return value
 
     @classmethod
-    def load_init_param(cls, config: "ComponentConfig", name: str, value: Any) -> Any:
+    def load_init_param(cls, config: "jembe.ComponentConfig", name: str, value: Any) -> Any:
         """
         load and Decode init/state param received via json call to be uset to initialise in __init__.
         param_value is decoded from json received from client.
@@ -744,14 +887,8 @@ class Component(metaclass=ComponentMeta):
             except ValueError as e:
                 if current_app.debug or current_app.testing:
                     raise JembeError(
-                        "State param {} of {}.{} with hint {} is not supported for json dump/load "
-                        "nor custom encoding/decoding logic is defined in dump_init_param/load_init_param. \n\n({}) ".format(
-                            name,
-                            cls.__module__,
-                            cls.__name__,
-                            cls._jembe_init_signature.parameters[name],
-                            e,
-                        )
+                        f"State param {name} of {cls.__module__}.{cls.__name__} with hint {cls._jembe_init_signature.parameters[name]} is not supported for json dump/load "
+                        f"nor custom encoding/decoding logic is defined in dump_init_param/load_init_param. \n\n({e}) "
                     )
         return value
 
@@ -835,7 +972,7 @@ class Component(metaclass=ComponentMeta):
             template = list(template)
         # add reference to itself in context (usefull for passing parameters in macros)
         context["_context"] = context
-        return render_template(template, **context)
+        return render_template(template, **context)  # type:ignore
 
     def render_template_string(self, source, **context):
         """
@@ -939,11 +1076,7 @@ class Component(metaclass=ComponentMeta):
 
     def _jinja2_placeholder(self, _jmb_copmonent_name: str):
         return Markup(
-            '<{dtag} jmb-placeholder-permanent="{exec_name}/{component_name}"></{dtag}>'.format(
-                exec_name=self.exec_name,
-                component_name=_jmb_copmonent_name,
-                dtag="template",
-            )
+            f'<template jmb-placeholder-permanent="{self.exec_name}/{_jmb_copmonent_name}"></template>'
         )
 
     def emit(self, _jmb_event_name: str, **params) -> "EmitCommand":
@@ -956,7 +1089,7 @@ class Component(metaclass=ComponentMeta):
         self,
         _jmb_component_name: str,
         _jmb_component_key: Optional[str] = None,
-        **init_params
+        **init_params,
     ):
         """
         Calls initCommand and displayComand of subcomponent without need to
@@ -964,31 +1097,12 @@ class Component(metaclass=ComponentMeta):
         """
         if _jmb_component_name not in self._config.components.keys():
             raise JembeError(
-                "Component '{}' is not valid sub component of '{}'!".format(
-                    _jmb_component_name, self._config.full_name
-                )
+                f"Component '{_jmb_component_name,}' is not valid sub component of '{self._config.full_name}'!"
             )
-        processor = get_processor()
-        cexec_name = "{}/{}".format(
-            self.exec_name,
-            _jmb_component_name
-            if _jmb_component_key is None
-            else "{}.{}".format(_jmb_component_name, _jmb_component_key),
-        )
-        processor.add_command(
-            InitialiseCommand(cexec_name, init_params),
-            end=True,
-        )
-        processor.add_command(
-            CallDisplayCommand(
-                cexec_name,
-                displayed_by_exec_name=self.exec_name,
-            ),
-            end=True,
-        )
-        processor.components_marked_for_removal = [
-            en for en in processor.components_marked_for_removal if en != cexec_name
-        ]
+        cr = self.component(_jmb_component_name, **init_params)
+        if _jmb_component_key:
+            cr.key(_jmb_component_key)
+        cr.execute()
 
     def remove_component(
         self, _jmb_component_name: str, _jmb_component_key: Optional[str] = None
@@ -1002,9 +1116,7 @@ class Component(metaclass=ComponentMeta):
         """
         if _jmb_component_name not in self._config.components.keys():
             raise JembeError(
-                "Component '{}' is not valid sub component of '{}'!".format(
-                    _jmb_component_name, self._config.full_name
-                )
+                f"Component '{_jmb_component_name}' is not valid sub component of '{self._config.full_name}'!"
             )
         processor = get_processor()
         cexec_name = "{}/{}".format(
@@ -1035,7 +1147,7 @@ class Component(metaclass=ComponentMeta):
         root_cr = component_ref.root_renderer
         processor.remove_component(
             root_cr.exec_name,
-            self.exec_name.startswith("{}/".format(root_cr.exec_name)),
+            self.exec_name.startswith("{root_cr.exec_name}/"),
         )
 
         # Creates new commands to init and display for redirect components
