@@ -1,3 +1,4 @@
+from re import T
 from typing import (
     Callable,
     Iterable,
@@ -5,6 +6,7 @@ from typing import (
     Optional,
     Type,
     Dict,
+    List,
     Tuple,
     Union,
     Sequence,
@@ -38,7 +40,7 @@ class UrlConvertor(Enum):
 
 
 def calc_url_param_identifier(name: str, level: int):
-    return name if level == 0 else "{}__{}".format(name, level)
+    return name if level == 0 else f"{name}__{level}"
 
 
 class UrlParamDef:
@@ -73,9 +75,7 @@ class UrlParamDef:
 
     @property
     def url_pattern(self) -> str:
-        return "<{convertor}:{identifier}>".format(
-            convertor=self.convertor.value, identifier=self.identifier
-        )
+        return f"<{self.convertor.value}:{self.identifier}>"
 
     def __repr__(self):
         return self.url_pattern
@@ -88,17 +88,51 @@ def action(
     # deferred_after: Optional[str] = None,
     # deferred_before: Optional[str] = None,
 ):
-    """
-    decorator to mark method as public action inside component
 
-    deferred aciton is executed last after allother actions from parent action template
-    are executed no matter of its postion inside parent action template.
+    # deferred aciton is executed last after allother actions from parent action template
+    # are executed no matter of its postion inside parent action template.
 
-    Usefull if we need to create breadcrumb or other summary report
-    based from already executed actions
+    # Usefull if we need to create breadcrumb or other summary report
+    # based from already executed actions
 
-    deferred_after and deferred_before are used to execute this action after or before
-    other specific deferred action, when multiple actions has deferred execution
+    # deferred_after and deferred_before are used to execute this action after or before
+    # other specific deferred action, when multiple actions has deferred execution
+    """Decorator to mark Component method as public action
+
+    Public actions can be called from browser using javascript, as follows:
+
+    .. code-block:: html
+
+        <button onclick="jembeClient.component(this).call('action',{param1:value1, param2:value2})">
+            Action
+        </button>
+
+        Or using Jembe Js ComponentApi:
+
+        <button jmb-on:click="action(value1)">
+            Action
+        </button>
+
+        or
+
+        <a href="" jmb-on:click.prevent="action()">
+            Action
+        </a>
+
+    If action is defered than it is executed last, after all other actions from parent action template
+    are executed no matter of its postion inside parent action template. This is usefull if we need
+    to create breadcrumb or other summary report based from already executed actions.
+
+    Args:
+        deferred (bool, optional): If action is deffered. Defaults to False.
+
+    Returns:
+        Optional[bool]: Decorated action should return None, True or False
+            If decorated action returns:
+
+            - True: then component will redisplay itself.
+            - False: then component will NOT redisplay itself.
+            - None: then component will redisplay it self based on values of Config.redisplay and state parameters. Defult if will redisplay itself only if state params are changed.
     """
     # This decorator don't anytthing except allow
     # componentconfig.set_compoenent_class to
@@ -122,24 +156,56 @@ def listener(
     event: Optional[Union[str, Sequence[str]]] = None,
     source: Optional[Union[str, Sequence[str]]] = None,
 ):
-    """
-    decorator to mark method as action listener inside component
-    filter by:
-    source = glob like component exec_name matcher
-    if source is:
+    """Decorator to mark Component method as event listener.
+
+    Event listener listens for emited events.
+
+    Event listener must accepet event parameter like:
+
+    .. code-block:: python
+
+        @listener("_display", source="*")
+        def on_display_any_direct_childred(self, event:"jembe.Event"):
+            pass
+
+    Args:
+        event (Optional[Union[str, Sequence[str]]], optional): Event name to listen for. Defaults to None.
+
+            If event name is:
+
+            - None: it will listen for any event
+            - str: it will listen for only event with the same name
+            - Sequence[str]: it will listen for any event name from the list of names
+
+        source (Optional[Union[str, Sequence[str]]], optional): Listens for events emited by component specified as source. Defaults to None.
+
+            Listen for:
+
+            - None -> events from any component
+            - *                               -> direct children with or without key
+            - \*.\*                             -> direct children with any key
+            - **/*                            -> all children with or without key
+            - **/\*.\*                          -> all children with any key
+            - component                       -> match to direct child named "component" without key
+            - component.*                     -> match to direct child named "component" with any key
+            - component.key                   -> match to direct child named "component with key equals "key"
+            - **/component[.[*|<key>]]        -> match to child at any level with "component" name
+            - ..                                -> match to parent component
+            - ../component[.[*|<key>]]          -> match to sibling
+            - /**/.                             -> match to parent at any level
+            - /\*\*/component[.[*|<key>]]/\*\*/.    -> match to parent at any level named "component"
+            - //                                -> process event from root/page component
+            - .                                 -> self
+
+    Returns:
+        Optional[bool]: Decorated method should return None, True or False
+
+            If decorated listener returns:
+
+            - True: then component will redisplay itself.
+            - False: then component will NOT redisplay itself.
+            - None: then component will redisplay it self based on values of Config.redisplay and state parameters. Defult if will redisplay itself only if state params are changed.
 
-        - None -> process event from any compoenent
-        - /compoent1.key1/compoent2.key2    -> process event from  compoenent with this exec_name
-        - ./component                       -> process event from direct child named "component" without key
-        - ./component.*                     -> process event from direct child named "component" with any key
-        - ./component.key                   -> process event from direct child named "component with key equals "key"
-        - ./**/component[.[*|<key>]]        -> process event from child at any level
-        - ..                                -> process event from parent
-        - ../component[.[*|<key>]]          -> process event from sibling
-        - /**/.                             -> process event from parent at any level
-        - /**/component[.[*|<key>]]/**/.    -> process event from parent at any level named
-        - //                                -> process event from root page
-        - etc.
     """
     # This decorator don't anytthing except allow
     # componentconfig.set_compoenent_class to
@@ -163,10 +229,15 @@ def redisplay(
     when_executed: Optional[bool] = None,
     when_on_page: Optional[bool] = None,
 ):
-    """
-    Decorates display method in order to set redisplay ComponentConfig param.
+    """Decorator for Component.display method to configure when component will be redisplayed
 
-    Made for easy use when configuring components
+    It's used instead of Component.Config redisplay parameter, when it's more convinient.
+
+
+    Args:
+        when_state_changed (Optional[bool], optional): Redisplay component whent its state is changed. Defaults to None.
+        when_executed (Optional[bool], optional): Redisplay component when ever "display" action is called regradles of the component state. Defaults to None.
+        when_on_page (Optional[bool], optional): Redisplay component whenever it's on the page. Defaults to None.
     """
 
     def decorator_action(method):
@@ -191,8 +262,19 @@ def redisplay(
 
 
 def config(component_config: "jembe.ComponentConfig"):
-    """
-    Decorates Component to chante its ComponentConfig init parameters
+    """Component decorator to declarativly change Component Configuration
+
+    This decorator changes Component Configuration by suplaying predefined
+    params to Component.ComponentConfig.__init__ method.
+
+    .. note::
+        ComponentConfig instance in argument is used only to get values of __init__ params.
+
+        Collected params are used when component is registred to Jembe instance,
+        allowing parent component to overide configuration of its subcomponents when needed.
+
+    Args:
+        component_config (jembe.ComponentConfig): Instance of Component.Config class
     """
 
     def decorator_class(component_class):
@@ -317,7 +399,7 @@ def componentConfigInitDecorator(init_method):
                         },
                     )
             except Exception as e:
-                print("Error in {} __init__: {}".format(kwargs, e))
+                print("Error in {kwargs} __init__: {e}")
                 raise e
         else:
             # Component config is used inside @config or @page decorator
@@ -377,7 +459,7 @@ class ComponentConfig(metaclass=ComponentConfigMeta):
         cconfig._component_class = _component_class
         cconfig._parent = _parent
         cconfig._jembe_prepare_component_init()
-        cconfig.__init__(**init_params)
+        cconfig.__init__(**init_params)  # type:ignore
         return cconfig
 
     def _jembe_prepare_component_init(self):
@@ -460,23 +542,35 @@ class ComponentConfig(metaclass=ComponentConfigMeta):
         changes_url: bool = True,
         url_query_params: Optional[Dict[str, str]] = None,
     ):
-        """
+        """_summary_
+
         template: path to default template for displaying component
         components: Definition of subcomponents
         inject_into_components: Callable to inject init params into subcomponents
         redisplay: Flag to define when componnet will be redisplayed on client depending of its state
         changes_url: Does this component changes location url and allow back browser navigation to it
         url_query_params: Mapping from GET Query params to state params used when component is called directly via regular http get request dict(<name of get queryparam> = <name of state param>)
+        Args:
+            template (Optional[Union[str, Iterable[str]]], optional): _description_. Defaults to None.
+            components (Optional[Dict[str, &quot;jembe.ComponentRef&quot;]], optional): _description_. Defaults to None.
+            inject_into_components (Optional[ Callable[[&quot;jembe.Component&quot;, &quot;jembe.ComponentConfig&quot;], dict] ], optional): _description_. Defaults to None.
+            changes_url (bool, optional): _description_. Defaults to True.
+            url_query_params (Optional[Dict[str, str]], optional): _description_. Defaults to None.
         """
+
         # use default template if template name is not provided
         if template is None:
-            self.template: Tuple[str, ...] = (self.default_template_name,)
+            self.template: Tuple[str, ...] = self.default_template_names
         elif isinstance(template, str):
             self.template = (template,)
         else:
-            self.template = tuple(
-                t if t != "" else self.default_template_name for t in template
-            )
+            templates: List[str] = []
+            for t in template:
+                if t == "":
+                    templates.extend(self.default_template_names)
+                else:
+                    templates.append(t)
+            self.template = tuple(templates)
 
         self.components: Dict[str, "jembe.ComponentRef"] = (
             components if components else dict()
@@ -501,7 +595,7 @@ class ComponentConfig(metaclass=ComponentConfigMeta):
         try:
             return self.__endpoint
         except AttributeError:
-            raise JembeError("Endpoint not set by jembe app: {}".format(self.full_name))
+            raise JembeError(f"Endpoint not set by jembe app: {self.full_name}")
 
     @endpoint.setter
     def endpoint(self, endpoint: str):
@@ -513,9 +607,7 @@ class ComponentConfig(metaclass=ComponentConfigMeta):
             return getattr(self, "_component_class")
         except AttributeError:
             raise JembeError(
-                "Component Config {} is not initialised properly, 'component_class' is missing.".format(
-                    self.full_name
-                )
+                f"Component Config {self.full_name} is not initialised properly, 'component_class' is missing."
             )
 
     @property
@@ -524,9 +616,7 @@ class ComponentConfig(metaclass=ComponentConfigMeta):
             return getattr(self, "_name")
         except AttributeError:
             raise JembeError(
-                "Component Config {} is not initialised properly, 'name' is missing.".format(
-                    self.full_name
-                )
+                f"Component Config {self.full_name} is not initialised properly, 'name' is missing."
             )
 
     @property
@@ -535,31 +625,23 @@ class ComponentConfig(metaclass=ComponentConfigMeta):
             return getattr(self, "_parent")
         except AttributeError:
             raise JembeError(
-                "Component Config {} is not initialised properly, 'parent' is missing.".format(
-                    self.full_name
-                )
+                f"Component Config {self.full_name} is not initialised properly, 'parent' is missing."
             )
 
     @property
     def full_name(self) -> str:
         if self.parent:
-            return "{}/{}".format(self.parent.full_name, self.name)
-        return "/{}".format(self.name)
+            return f"{self.parent.full_name}/{self.name}"
+        return f"/{self.name}"
 
     @property
     def url_path(self) -> str:
         if not self._url_params:
-            url_path = "/{}{}".format(self.name, self._key_url_param.url_pattern)
+            url_path = f"/{self.name}{self._key_url_param.url_pattern}"
         else:
             url_params = "/".join(up.url_pattern for up in self._url_params)
-            url_path = "/{name}{key}/{url_params}".format(
-                name=self.name,
-                key=self._key_url_param.url_pattern,
-                url_params=url_params,
-            )
-        url_path = (
-            "{}{}".format(self.parent.url_path, url_path) if self.parent else url_path
-        )
+            url_path = f"/{self.name}{self._key_url_param.url_pattern}/{url_params}"
+        url_path = f"{self.parent.url_path}{url_path}" if self.parent else url_path
         return url_path
 
     @property
@@ -572,7 +654,7 @@ class ComponentConfig(metaclass=ComponentConfigMeta):
     def get_raw_url_params(self, state: "ComponentState", key: str) -> dict:
         url_params = {up.identifier: state[up.name] for up in self._url_params}
         url_params[self._key_url_param.identifier] = (
-            "{}{}".format(self.KEY_URL_PARAM_SEPARATOR, key) if key else ""
+            f"{self.KEY_URL_PARAM_SEPARATOR}{key}" if key else ""
         )
         return url_params
 
@@ -596,8 +678,9 @@ class ComponentConfig(metaclass=ComponentConfigMeta):
         return url_for(self.endpoint, **url_params)
 
     @property
-    def default_template_name(self) -> str:
-        return "{}.html".format(self.full_name.strip("/"))
+    def default_template_names(self) -> Tuple[str, ...]:
+        tname = f"{self.full_name.strip('/')}.html"
+        return (tname, f"pages/{tname}")
 
     def update_components_config(
         self,
@@ -635,9 +718,7 @@ class ComponentConfig(metaclass=ComponentConfigMeta):
         else:
             if name not in self.components:
                 raise ValueError(
-                    "Component with name {} is not defined inside {}".format(
-                        name, self.name
-                    )
+                    f"Component with name {name} is not defined inside {self.name}"
                 )
             _update_cref(name, params)
 
@@ -649,7 +730,7 @@ class ComponentConfig(metaclass=ComponentConfigMeta):
         configs: Dict[str, "jembe.ComponentConfig"] = dict()
         for component_name in self.components.keys():
             configs[component_name] = processor.jembe.components_configs[
-                "{}/{}".format(self.full_name, component_name)
+                f"{self.full_name}/{component_name}"
             ]
         return configs
 
