@@ -72,6 +72,11 @@ class File(JembeInitParamSupport):
 
     @property
     def url(self) -> str:
+        if self.in_private_storage():
+            if self.is_cache_version():
+                self.get_original().grant_access()
+            else:
+                self.grant_access()
         return url_for(
             "jembe./jembe/file",
             component_key="",
@@ -118,6 +123,7 @@ class File(JembeInitParamSupport):
 
     def move_to_private(self, subdir: str = ""):
         file = self.move_to(get_private_storage(), subdir)
+        file.grant_access()
         self.storage = file.storage
         self.path = file.path
 
@@ -203,6 +209,9 @@ class File(JembeInitParamSupport):
             return False
         return self.storage == o.storage and self.path == o.path
 
+    def __hash__(self) -> int:
+        return hash((self.storage, self.path))
+
 
 class Storage(ABC):
     class Type(Enum):
@@ -265,22 +274,20 @@ class Storage(ABC):
         try:
             if file_path not in session[JEMBE_FILES_ACCESS_GRANTED][self.name]:
                 session[JEMBE_FILES_ACCESS_GRANTED][self.name].insert(0, file_path)
-                session.modified = True
+                if (
+                    len(session[JEMBE_FILES_ACCESS_GRANTED])
+                    > JEMBE_FILES_ACCESS_GRANTED_MAX_SIZE
+                ):
+                    session[JEMBE_FILES_ACCESS_GRANTED] = session[
+                        JEMBE_FILES_ACCESS_GRANTED
+                    ][:JEMBE_FILES_ACCESS_GRANTED_MAX_SIZE]
 
-            if (
-                len(session[JEMBE_FILES_ACCESS_GRANTED])
-                > JEMBE_FILES_ACCESS_GRANTED_MAX_SIZE
-            ):
-                session[JEMBE_FILES_ACCESS_GRANTED] = session[
-                    JEMBE_FILES_ACCESS_GRANTED
-                ][:JEMBE_FILES_ACCESS_GRANTED_MAX_SIZE]
                 session.modified = True
         except KeyError:
             if JEMBE_FILES_ACCESS_GRANTED not in session:
-                session[JEMBE_FILES_ACCESS_GRANTED] = dict()
-            if self.name not in session[JEMBE_FILES_ACCESS_GRANTED]:
-                session[JEMBE_FILES_ACCESS_GRANTED][self.name] = list()
-            session[JEMBE_FILES_ACCESS_GRANTED][self.name].insert(0, file_path)
+                session[JEMBE_FILES_ACCESS_GRANTED] = {self.name: [file_path]}
+            else:
+                session[JEMBE_FILES_ACCESS_GRANTED][self.name] = [file_path]
             session.modified = True
 
     def revoke_access_to_file(self, file_path: str):
@@ -431,7 +438,7 @@ class Storage(ABC):
         return File(storage=self, file_path=full_path)
 
     def get_original_file(self, cache_file_path: str) -> "jembe.File":
-        if cache_file_path.startswith("{}/".format(DEFAULT_STORAGE_CACHE_FOLDER)):
+        if not cache_file_path.startswith("{}/".format(DEFAULT_STORAGE_CACHE_FOLDER)):
             raise ValueError(
                 "Invalid cache file name '{}' in storage '{}'".format(
                     cache_file_path, self.name
@@ -557,6 +564,9 @@ class Storage(ABC):
             return False
         return self.name == o.name and self.type == o.type
 
+    def __hash__(self) -> int:
+        return hash((self.name, self.type))
+
 
 class DiskStorage(Storage):
     """Stores files on disk"""
