@@ -27,6 +27,7 @@ from jinja2 import Undefined
 from lxml import etree
 from lxml.html import Element
 from flask import json, jsonify, g
+from werkzeug.exceptions import NotFound
 from werkzeug import Response
 from .common import (
     exec_name_to_full_name,
@@ -38,7 +39,7 @@ from .common import (
     # json_object_hook,
     # json_default,
 )
-from .exceptions import AccessDenied, Forbidden, JembeError, NotFound, Unauthorized
+from .exceptions import AccessDenied, Forbidden, JembeError, Unauthorized
 from .component_config import ComponentConfig, RedisplayFlag as RedisplayFlag
 
 
@@ -198,9 +199,10 @@ class CallActionCommand(Command):
             # Do nothing
             pass
         elif isinstance(action_result, Response):
-            # TODO If self.component is component directly requested via http request
-            # and it is not x-jembe request return respon
-            # othervise raise JembeError
+            # TODO return response and
+            # stop fruther processiong of requst with jembe processor
+            # TODO jembe.js should be able to handle direct reponses
+            # TODO make the same for listener
             raise NotImplementedError()
         else:
             raise JembeError(
@@ -407,6 +409,13 @@ class CallListenerCommand(Command):
         elif isinstance(listener_result, bool) or listener_result == False:
             # Do nothing
             pass
+        elif isinstance(listener_result, Response):
+            # TODO return response and
+            # stop fruther processiong of requst with jembe processor
+            # TODO jembe.js should be able to handle direct reponses
+            # TODO make the same for action
+            # raise NotImplementedError()
+            return listener_result
         else:
             raise JembeError(
                 "Invalid listener result type: {}.{} {}".format(
@@ -678,7 +687,6 @@ class EmitCommand(Command):
             if not destination_listener.sources
             else destination_listener.sources
         )
-
         for s_to in sources_to:
             for dl_event in listener_events:
                 for dl_source in listener_sources:
@@ -1213,6 +1221,9 @@ class Processor:
                 )
             exec_names.append(exec_name)
 
+        if exec_name not in self.request.path:
+            # dont allow that /mainaaaa open /main.aaaa
+            raise NotFound()
         return exec_names
 
     def process_request(self) -> "Processor":
@@ -1421,7 +1432,9 @@ class Processor:
         exception is already handled by its children.
 
         Parent component can use regular listener to catch event do whatever thay want. @listener('_exception')
-        If any of parrent compoennt mark event as handled event.params["handled"] = True exception will not be raised again.
+        If any of parrent component mark event as handled event.params["handled"] = True exception will not be raised again.
+        IMPORTANT: When parent component mark exception as handled all its subcomponents and events will be removed from processor
+        so that parent component can rerender itself and/or its subcomponents from the blank state
 
         If there is not listener for _exception event in parent compoent or exception event is not handled. Processor
         will rerise exception and it will be handled like regular flask exception.
@@ -1468,13 +1481,13 @@ class Processor:
                     self._raised_exception_on_initialise[
                         command.component_exec_name
                     ] = deepcopy(command.init_params)
-                    # remove all child compoennt of command.component_exec_name
+                    # remove all child components of command.component_exec_name
                     self.components = {
                         exec_name: comp
                         for exec_name, comp in self.components.items()
                         if not is_child_name(cmd.component_exec_name, exec_name)
                     }
-                    # remove all commands to and from  removed components
+                    # remove all commands to and from removed components
                     self._commands = deque(
                         (
                             c
@@ -1492,8 +1505,10 @@ class Processor:
 
         # if exception is not handled by parent components
         # rerise exception
-        current_app.logger.error("Unhandled exception in {}: {}".format(command, exc))
-        if current_app.debug or current_app.testing:
+        if current_app.debug or current_app.testing or not isinstance(exc, NotFound):
+            current_app.logger.error(
+                "Unhandled exception in {}: {}".format(command, exc)
+            )
             import traceback
 
             traceback.print_exc()
